(document:surround "/std/frame")

(define (group)
  (selector-id value))

(define (apply-to-backend)
  (woo-catch/message (thunk
    (woo-write "/groups/"
      'group (group)
))))

(define (reset-from-backend)
  (woo-catch/message (thunk
    (let* ((data (woo-read-first "/groups/"
          'name  (if (= 0 (selector-id count)) "" (group))))
        (name  (woo-get-option data 'name ""))
        (gid   (woo-get-option data 'gid ""))
        (label (woo-get-option data 'label ""))
        (users (woo-get-option data 'users ""))
        (prot  (woo-get-option data 'protected #t)) )
      (name-id  text name)
      (gid-id   text gid)
      (label-id text label)
      (update-member-list name)
      (delete-btn activity (not prot))
))))

(define (reset-list)
  (selector-id (enumref "/groups/"))
  (if (> (selector-id count) 0) (selector-id current 0))
  (reset-from-backend)
)

(define (reset-skippings)
  (woo-catch/message (thunk
    (woo-try "set_skippings" "/groups/"
      'show_system (show-system-groups value)
      'show_user   (show-user-groups value)
  )))
  (reset-list)
)

(define (group-del)
  (woo-catch/message (thunk
    (woo-try "delete" "/groups" 'name (selector-id value))))
  (reset-list)
)

(define (group-add)
  (if (woo-catch/message (thunk
    (woo-try "new" "/groups" 'name (new-id value))))
    (begin
      (reset-skippings)
      (selector-id value (new-id value))
      (new-id value "")
      (reset-from-backend)
    )))

;;; Membership management

(define (update-member-list group)
  (form-update-enum "members" (woo-list "/groups/members" 'group group))
  (form-update-enum "non-members" (woo-list "/groups/non_members" 'group group))
)

(define (member-add)
  (let ((user (form-value "non-members")))
    (and (string? (group)) (string? user)
         (catch/message
           (lambda()
             (woo-write "/groups/add_member"
                  'language (form-value "language")
                  'group (group)
                  'user  user)
             (update-member-list (group)))))))

(define (member-remove)
  (let ((user (form-value "members")))
    (and (string? (group)) (string? user)
         (catch/message
           (lambda()
             (woo-write "/groups/remove_member"
                  'language (form-value "language")
                  'group (group)
                  'user  user)
             (update-member-list (group)))))))

;;; User interface

(gridbox columns "10;80;10"

  (spacer)
  (gridbox columns "0;100;0"
    (label text (_ "New group:" ))
    (document:id new-id (edit))
    (button text (_ "Create") (when clicked (group-add))) )
  (spacer)

  (spacer)
  (gridbox columns "40;60"

    (document:id selector-id (listbox
      (when selected (reset-from-backend))))

    (gridbox columns "100"
     (gridbox columns "0;100"
        (label text (_ "Name:") align "right")
        (document:id name-id (label text "" text-wrap #t))
        (label text (_ "GID:") align "right")
        (document:id gid-id (label text "" text-wrap #t))
        (label text (_ "Description:") align "right")
        (document:id label-id (label text "" text-wrap #t))
      )
      (groupbox title (_ "User membership")
	(gridbox columns "45;10;45"
          (label text (_ "Members:"))
          (spacer)
          (label text (_ "Available users:"))
	  (listbox name "members")
	  (gridbox columns "100"
	    (spacer)
	    (button text (_ "<<") (when clicked (member-add)))
	    (button text (_ ">>") (when clicked (member-remove)))
	    (spacer)
	  )
	 (listbox name "non-members")
	)
      )

      (gridbox columns "33;33;33"
        (button text (_ "Apply")  (when clicked (apply-to-backend) (reset-from-backend)))
        (document:id delete-btn (button text (_ "Delete") (when clicked (group-del))))
      )
      (spacer)
    )

    (gridbox columns "100"
      (document:id show-system-groups (checkbox (_ "Show system groups")
        value #t (when toggled (reset-skippings))))
      (document:id show-user-groups (checkbox (_ "Show user groups")
        value #f (when toggled (reset-skippings))))
      (spacer)
    )
    (spacer)
  )
  (spacer)
  (spacer)
  (groupbox title (_ "Notes")
    (label text (_ "Groups for system services (with prefix '_') are not shown. You can delete only user groups.")))
  (spacer)
)

(document:root (when loaded (reset-list)))
